package jdk.internal.org.objectweb.asm.tree;

import java.util.ArrayList;
import java.util.Map;
import jdk.internal.org.objectweb.asm.MethodVisitor;
import java.util.List;

public abstract class AbstractInsnNode
{
    public static final int INSN = 0;
    public static final int INT_INSN = 1;
    public static final int VAR_INSN = 2;
    public static final int TYPE_INSN = 3;
    public static final int FIELD_INSN = 4;
    public static final int METHOD_INSN = 5;
    public static final int INVOKE_DYNAMIC_INSN = 6;
    public static final int JUMP_INSN = 7;
    public static final int LABEL = 8;
    public static final int LDC_INSN = 9;
    public static final int IINC_INSN = 10;
    public static final int TABLESWITCH_INSN = 11;
    public static final int LOOKUPSWITCH_INSN = 12;
    public static final int MULTIANEWARRAY_INSN = 13;
    public static final int FRAME = 14;
    public static final int LINE = 15;
    protected int opcode;
    public List<TypeAnnotationNode> visibleTypeAnnotations;
    public List<TypeAnnotationNode> invisibleTypeAnnotations;
    AbstractInsnNode prev;
    AbstractInsnNode next;
    int index;
    
    protected AbstractInsnNode(int opcode) {
        this.opcode = opcode;
        this.index = -1;
    }
    
    public int getOpcode() {
        return this.opcode;
    }
    
    public abstract int getType();
    
    public AbstractInsnNode getPrevious() {
        return this.prev;
    }
    
    public AbstractInsnNode getNext() {
        return this.next;
    }
    
    public abstract void accept(MethodVisitor p0);
    
    protected final void acceptAnnotations(MethodVisitor methodVisitor) {
        for (int n = (this.visibleTypeAnnotations == null) ? 0 : this.visibleTypeAnnotations.size(), i = 0; i < n; ++i) {
            TypeAnnotationNode typeAnnotationNode = (TypeAnnotationNode)this.visibleTypeAnnotations.get(i);
            typeAnnotationNode.accept(methodVisitor.visitInsnAnnotation(typeAnnotationNode.typeRef, typeAnnotationNode.typePath, typeAnnotationNode.desc, true));
        }
        for (int n2 = (this.invisibleTypeAnnotations == null) ? 0 : this.invisibleTypeAnnotations.size(), j = 0; j < n2; ++j) {
            TypeAnnotationNode typeAnnotationNode2 = (TypeAnnotationNode)this.invisibleTypeAnnotations.get(j);
            typeAnnotationNode2.accept(methodVisitor.visitInsnAnnotation(typeAnnotationNode2.typeRef, typeAnnotationNode2.typePath, typeAnnotationNode2.desc, false));
        }
    }
    
    public abstract AbstractInsnNode clone(Map<LabelNode, LabelNode> p0);
    
    static LabelNode clone(LabelNode labelNode, Map<LabelNode, LabelNode> map) {
        return (LabelNode)map.get(labelNode);
    }
    
    static LabelNode[] clone(List<LabelNode> list, Map<LabelNode, LabelNode> map) {
        LabelNode[] array = new LabelNode[list.size()];
        for (int i = 0; i < array.length; ++i) {
            array[i] = (LabelNode)map.get(list.get(i));
        }
        return array;
    }
    
    protected final AbstractInsnNode cloneAnnotations(AbstractInsnNode abstractInsnNode) {
        if (abstractInsnNode.visibleTypeAnnotations != null) {
            this.visibleTypeAnnotations = new ArrayList();
            for (int i = 0; i < abstractInsnNode.visibleTypeAnnotations.size(); ++i) {
                TypeAnnotationNode typeAnnotationNode = (TypeAnnotationNode)abstractInsnNode.visibleTypeAnnotations.get(i);
                TypeAnnotationNode o = new TypeAnnotationNode(typeAnnotationNode.typeRef, typeAnnotationNode.typePath, typeAnnotationNode.desc);
                typeAnnotationNode.accept(o);
                this.visibleTypeAnnotations.add(o);
            }
        }
        if (abstractInsnNode.invisibleTypeAnnotations != null) {
            this.invisibleTypeAnnotations = new ArrayList();
            for (int j = 0; j < abstractInsnNode.invisibleTypeAnnotations.size(); ++j) {
                TypeAnnotationNode typeAnnotationNode2 = (TypeAnnotationNode)abstractInsnNode.invisibleTypeAnnotations.get(j);
                TypeAnnotationNode o2 = new TypeAnnotationNode(typeAnnotationNode2.typeRef, typeAnnotationNode2.typePath, typeAnnotationNode2.desc);
                typeAnnotationNode2.accept(o2);
                this.invisibleTypeAnnotations.add(o2);
            }
            return this;
        }
        return this;
    }
}
